// sapienexe.cpp : send commands to WowWees through LEGO IR tower, executable version (Win32 & Linux)

#include "sapien.h"

int main(int argc, char* argv[])
{
  int i;
  HANDLE f;

  if (argc == 2)
    if ((strcmp(argv[1], "-?") == 0) ||
        (strcmp(argv[1], "-h") == 0) ||
		(strcmp(argv[1], "-help") == 0) ||
		(strcmp(argv[1], "/help") == 0) ||
        (strcmp(argv[1], "--help") == 0) ||
        (strcmp(argv[1], "/?") == 0)) argc = 1;

  if (argc == 1)
  {
      printf("usage: sapien cmd|code|-script filename [-port portname] [-nowakeup] [-v2|-pet] [-rcx]\n\n");
      printf("        codes are specified in hexadecimal as 0xCC\n");
      printf("        specify -v2 for Sapien V2 or RoboRaptor, and -pet for RoboPet,\n");
      printf("         default is RoboSapien v1.\n");
	  printf("        specify -rcx to relay command through RCX\n");
      printf("        script should have 1 cmd/line preceeded by time in milliseconds\n");
      printf("         and space; empty lines and those starting with # are ingored\n");
      printf("        to view the list of commands for RoboSapien, run sapien -commands\n");
      printf("        to view the list of commands for RoboSapien V2, run sapien -commandsv2\n");
      printf("        to view the list of commands for RoboPet, run sapien -commandspet\n");
	  printf("        to view the list of commands for RCX, run sapien -commandsrcx\n");
      return 0;
  }

  if (argc == 2)
  {
    if (strcmp(argv[1], "-commands") == 0)
    {
      printf("\nThe following commands are recognized for RoboSapien:\n");
      for (i = 0; i < NCMDSAPIEN; i++)
        if (cmd[i][0] != '-') printf("           %s (%x)\n", cmd[i], 128 + i);
      return 0;
    }
    else if (strcmp(argv[1], "-commandsv2") == 0)
    {
      printf("\nThe following commands are recognized for RoboSapien V2:\n");
      for (i = 0; i < NCMDSAPIEN2; i++)
        if (cmd2[i][0] != '_') printf("           %s (%x)\n", cmd2[i], codes2[i]);
        else printf("           DX%s (%x)\n", cmd2[i], codes2[i]);
      printf("\nIn the commands starting with DX, replace X with direction: \n");
      printf("  0 - Up/Forward\n  1 - Down/Backward\n  2 - Left\n  3 - Right\n  4 - Up and Left\n  5 - Up and Right\n  6 - Down and Left\n");
      printf("  7 - Down and Right\n");
      return 0;
    }
    else if (strcmp(argv[1], "-commandspet") == 0)
    {
      printf("\nThe following commands are recognized for RoboPet:\n");
      for (i = 0; i < NCMDPET; i++)
        printf("           %s (%x)\n", petcmd[i], petcodes[i]);
      return 0;
    }
	else if (strcmp(argv[1], "-commandsrcx") == 0)
	{
      printf("\nIf you specify -rcx, you can use the same commands as in default mode.\n");
	  printf("(run sapien with -commands, -commandsv2, or -commandspet options for list.)\n");
	  printf("With -rcx, you can also control RCX motors with the rcx motor command:\n");
	  printf("\nrcx_<motors>_<direction>[_<speed>]\n");
	  printf("\nexplanation:\n");
	  printf("  <motors> is a sequence of any of characters A,B,C representing the list\n");
	  printf("           of motors that should be affected by the command\n");
	  printf("  <direction> is one of 'fwd', 'rev', 'off', or 'brake'\n");
	  printf("  <speed> part is optional and is a decadic number 0-255. If not specified,\n");
	  printf("          the maximum speed will be used (255).\n");
	  printf("\nexamples:\n\n");
      printf("  rcx_AC_fwd_127   will start moving motors A and C forward half speed\n");
	  printf("  rcx_ABC_off      will stop all motors (put them to a loose mode)\n");
	  printf("  rcx_ABC_brake    will brake all motors immediatelly (uses power, should\n");
	  printf("                    be followed by off!)\n");
	  printf(" the sequence of commands:\n");
	  printf("  rcx_A_rev_127\n");
	  printf("  rcx_C_fwd_127    will start turning motor A in reverse direction and motor\n");
	  printf("                   C in forward direction half speed, in effect turning your\n");
	  printf("                   rower left\n");
	  printf("\nFor RCX memory bank programming, use the following commands:\n\n");
	  printf("  rcx_prgm_start[_<bank>]   where <bank> is an optional\n");
	  printf("                            decimal number 0-15\n");
	  printf("  rcx_prgm_store\n");
	  printf("  rcx_delay_XXXXX           where XXXXX is number of milliseconds\n");
	  printf("  rcx_prgm_run[_<bank>]\n");
	  return 0;
	}
  }

  int nowakeup = 0;
#ifdef WIN32
  char *outport = "COM1";
#else
  char *outport = "/dev/ttyS0";
#endif
  int version = 1;
  int rcx = 0;

  for (i = 2; i < argc; i++)
    if (strcmp(argv[i], "-port") == 0)
    {
      if (i < argc - 1) outport = argv[++i];
#ifdef WIN32
	  if (strcmp(outport, "usb") == 0)
		  outport = "\\\\.\\legotower1";
#else
	  outport = "usb_on_linux_not_supported_sorry";
#endif	 
    }
    else if (strcmp(argv[i], "-v2") == 0)
      version = 2;
    else if (strcmp(argv[i], "-pet") == 0)
      version = 3;
    else if (strcmp(argv[i], "-nowakeup") == 0)
      nowakeup = 1;
	else if (strcmp(argv[i], "-rcx") == 0)
	  rcx = 1;

  char *filename = 0;
  int code = 0;
  if ((argv[1][0] == '0') && (argv[1][1] == 'x'))
  {
      sscanf(argv[1] + 2, "%x", &code);
      if ((code > 0xFF) && (version == 1)) version = 2;
  }
  else if (strcmp(argv[1], "-script") != 0)
  {
      if (!(code = sap_decode(argv[1], version)))
          return 1;
  }
  else if (argc > 2) filename = argv[2];
  else
  {
      printf("sapien: missing script file name after -script\n");
      return 1;
  }

  if (rcx) f = rcx_init(outport, 0);
  else f = sap_open(outport);
  if (f == INVALID_HANDLE_VALUE) return 1;

  if (!filename) 
  {
	  if (rcx) rcx_sap_send(f, code);
	  else sap_send(f, 0, code);
  }
  else
  {
      FILE *g = fopen(filename, "r");
      if (!g)
      {
          printf("sapien: error opening file %s, errno=%d\n", filename, errno);
          return 1;
      }
      char buf[52];
      long t;

	  int run = 0;
      while (fgets(buf, 50, g))
      {
          if (buf[strlen(buf) - 1] == '\n') buf[strlen(buf) - 1] = '\0';
          if (strlen(buf) == 0) continue;
          if (buf[0] == '#') continue;
          char command[50];
          sscanf(buf, "%ld %s", &t, command);
#ifdef WIN32
          Sleep(t);
#else
          struct timespec tm;
          tm.tv_sec = (time_t)(t / 1000);
          tm.tv_nsec = (long)(t % 1000) * (long)1000000;
          nanosleep(&tm, 0);
#endif
          if ((command[0] == '0') && (command[1] == 'x'))
              sscanf(command + 2, "%x", &code);
          else code = sap_decode(command, version);
          if (rcx) rcx_sap_send(f, code);
		  else sap_send(f, run, code);
		  run++;
      }

      fclose(g);
  }

  sap_close(f);
  return 0;
}
